/*
 * File: Circle.c
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h> // for M_PI
#include "Shape.h"
#include "ShapeProtected.h"
#include "Circle.h"

struct CircleCDT {
	Shape shape;
	float radius;
};

Shape CircleShape(Circle circle) { return circle->shape; }

// We access these functions only through the "shape" interface, so they are unavailable to the user directly
static float CircleArea(void *instance) {
	Circle circle = (Circle) instance;
	return M_PI*(circle->radius)*(circle->radius);
}

static float CirclePerimeter(void *instance) {
	Circle circle = (Circle) instance;
	return 2.0*M_PI*(circle->radius);
}

static void CircleDraw(void *instance, int color) {
	Circle circle = (Circle) instance;
	printf("I am a circle of radius %f at (%f, %f) drawn with color %d!\n",
		circle->radius, circle->shape->x, circle->shape->y, color);
}

// This destructs only the derived additions
static void CircleDestruct(void *instance) {
	Circle circle = (Circle) instance;
	Shape shape = circle->shape;
	#ifndef QUIET_DTOR
		printf("Destructing circle at (%f,%f)\n", shape->x, shape->y);
	#endif
	shape->vft->obj_counter--;
	// Could be more work, if anything extra was allocated in "CircleCreate"
	free(circle);
	// Keep "shape", this method destructs only the "derived" stuff
}

/*
 * Function that should never be used because it puts us in a world of pain.
 * This invalidates the Shape interface contained in the Circle.
 * If there are any other references to this Shape,
 * there is no way for them to be updated to have a NULL value!
 */ 
void CircleDelete(Circle *circlePtr) {
	Shape *shapePtr = &((*circlePtr)->shape);
	ShapeDelete(shapePtr);
	*circlePtr = NULL;
}

static struct ShapeClassMembers CircleVFT = { 
	.class_name = "Circle",
	.obj_counter = 0, // not constant!
	.area = CircleArea, 
	.perimeter = CirclePerimeter,
	.draw = CircleDraw,
	.destruct = CircleDestruct
};

Circle CircleCreate(Shape shape, float radius) {
	Circle circle = malloc(sizeof(struct CircleCDT));
	// Attach the base (this must be in every derived class)
	if (shape) circle->shape = shape;
	else { fprintf(stderr,"error: base not initalized in CircleCreate\n"); exit(1); }
	shape->instance = circle;
	// Overload the inherited members (virtual functions)
	shape->vft = &CircleVFT;
	// Done with preliminaries! Now, the proper initialization
	// 1. Do whatever is needed with derived class members
	shape->vft->obj_counter++;
	// 2. Initialize the derived object members
	circle->radius = radius;
	#ifndef QUIET_CTOR
		printf("Creating a Circle at (%f, %f) of radius=%f\n", shape->x, shape->y, radius);
	#endif
	return circle;
}

// Circle methods

int CircleCounter(void) { return CircleVFT.obj_counter; }
float CircleGetRadius(Circle circle) { return circle->radius; }
void CircleSetRadius(Circle circle, float new_radius) { circle->radius = new_radius; }

