/*
 * File: Rectangle.c
 */

#include <stdio.h>
#include <stdlib.h>
#include "Shape.h"
#include "ShapeProtected.h"
#include "Rectangle.h"

struct RectangleCDT {
	Shape shape;
	float width, height;
};

Shape RectangleShape(Rectangle rect) { return rect->shape; }

// We access these functions only through the "shape" interface
static float RectangleArea(void *instance) {
	Rectangle rect = (Rectangle) instance;
	return (rect->width)*(rect->height);
}

static float RectanglePerimeter(void *instance) {
	Rectangle rect = (Rectangle) instance;
	return 2.0*((rect->width)+(rect->height));
}

static void RectangleDraw(void *instance, int color) {
	Rectangle rect = (Rectangle) instance;
	printf("I am a rectangle of width %f and height %f at (%f, %f) drawn with color %d!\n",
		rect->width, rect->height, rect->shape->x, rect->shape->y, color);
}

static void RectangleDestruct(void *instance) {
	Rectangle rect = (Rectangle) instance;
	Shape shape = rect->shape;
	#ifndef QUIET_DTOR
		printf("Destructing rectangle at (%f,%f)\n", shape->x, shape->y);
	#endif
	shape->vft->obj_counter--;
	// Could be more work, if anything extra was allocated in "RectangleCreate"
	free(rect);
	// Keep "shape", this method destructs only the "derived" stuff
}

static struct ShapeClassMembers RectangleVFT = { 
	.class_name = "Rectangle",
	.obj_counter = 0,
	.area = RectangleArea, 
	.perimeter = RectanglePerimeter,
	.draw = RectangleDraw,
	.destruct = RectangleDestruct
};

Rectangle RectangleCreate(Shape shape, float width, float height) {
	Rectangle rect = malloc(sizeof(struct RectangleCDT));
	// Attach the base (this must be in every derived class)
	if (shape) rect->shape = shape;
	else { fprintf(stderr,"error: base not initalized in RectangleCreate\n"); exit(1); }
	shape->instance = rect;
	// Overload the inherited members
	shape->vft = &RectangleVFT;
	shape->vft->obj_counter++;
	// Initialize the derived members
	rect->width = width; rect->height = height;
	#ifndef QUIET_CTOR
		printf("Creating a Rectangle at (%f, %f) of width=%f, height=%f\n", shape->x, shape->y, width, height);
	#endif
	return rect;
}

// Rectangle methods

int RectangleCounter(void) { return RectangleVFT.obj_counter; }
float RectangleGetWidth(Rectangle rect) { return rect->width; }
void RectangleSetWidth(Rectangle rect, float new_width) { rect->width = new_width; }
// we can arbitrarily make the "height" inaccessible to the user :)

