/*
 * File: Shape.h
 * -------------
 * The base class: the Shape interface
 * While the internals of implementation are hidden in "ShapeProtected.h" and are available
 * only to the derived classes, the interface functions are available to the user (public)
 */

#ifndef SHAPE_H
#define SHAPE_H

#include <stdbool.h>

// A shortcut for the handle
typedef struct ShapeInterfaceT *Shape;

// Constructor and destructor
// "New" calls should match "Delete" + "Replace" calls to prevent memory leaks
Shape ShapeNew(float x, float y);
void ShapeDelete(Shape *shapePtr); // sets *shapePtr to NULL
// Equivalent of copy-assignment operator in C++:
void ShapeReplace(Shape *old_shapePtr, Shape new_shape);

// To check if it is a derived class (DEBUGGING ONLY!)
bool ShapeIsVirtual(Shape shape);

// Interface functions
// 1. Getters and setters for data members
float ShapeGetX(Shape shape);
float ShapeGetY(Shape shape);
void ShapeSetX(Shape shape, float new_x);
void ShapeSetY(Shape shape, float new_y);
// .. and for class members
char *ShapeGetClassName(Shape shape); // to demonstrate a class data member, NOT to do any dispatch over subclasses!
int ShapeCounterOfThisType(Shape shape); // virtual class method (?) Determines class by an instance.
int ShapeCounterTotal(void); // class method
// 2. Methods (interfaces to overloaded virtual methods)
float ShapeArea(Shape shape);
float ShapePerimeter(Shape shape);
void ShapeDraw(Shape shape,int color);
// A method which is not a bare interface
void ShapeTranslation(Shape shape, float dx, float dy);

#endif // SHAPE_H

