/*
 * File: ShapeProtected.h
 * ----------------------
 * The implementation of the base class: the Shape interface.
 * The "pimpl" in C++ jargon :)
 * Only needs to be accessible to the derived classes, but not the user.
 */

#ifndef SHAPE_PROTECTED_H
#define SHAPE_PROTECTED_H

// Virtual function table.
// For each derived class we only have a single table with all class members
// and an object has only a pointer to it.
struct ShapeClassMembers {
	char *class_name; // class data member ("static" in C++)
	int obj_counter;
	float (*area)(void* instance);
	float (*perimeter)(void* instance);
	void (*draw)(void* instance, int color);
	void (*destruct)(void *instance);
};

// The implementation struct must be available to derived classes
struct ShapeInterfaceT {
	// 1. The access to the derived object, or equivalently, the object using the interface
	void *instance; // must be void* because we don't know its type yet
	// Interface elements
	// 2. Object members (usually, data members which are non-virtual by their nature)
	float x, y;
	// 3. Class members: data and methods, all collected in a virtual function table
	struct ShapeClassMembers *vft;
};

#endif // SHAPE_PROTECTED_H
